# $Id: jeff_char.py,v 1.4 2012-10-03 01:19:47 jdeifik Exp $
# Copyright Jeff turbo Deifik 2008
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# May-22-2012	JTD	Split from jefflib


import string			# string.join
import bisect			# Start2_ok, Finish_2_ok
import jefflib			# Print_Error


# CHARACTER FUNCTIONS

# Return True if character is a vowel
def Is_Vowel(ch):
    vowel = set(['a', 'e', 'i', 'o', 'u'])
    ch = ch.lower()
    for v in vowel:
        if ch == v:
            return True
    return False


# Return True if character is a blank
# Blanks include space, tab, cr, lf, ff, bs
def Is_Blank(ch):
    blank = set([' ', '\t', '\r', '\n', '\f', '\b'])
    ch = ch.lower()
    for v in blank:
        if ch == v:
            return True
    return False


# STRING TO LIST CONVERSION

# Takes in a list, returns a string
def List_to_String(lis):
    return str.join('', lis)


# Takes in a string, returns a list
def String_to_List(stir):
    liszt = []				# List of characters
    for c in stir:
        liszt.append(c)
    return liszt


# STRING FUNCTIONS

# This function os copyright Luca Dentis
# from http://code.activestate.com/recipes/498181/
# The license is psf, the python software foundation
# Takes in strings
#
# tSep is the thousands' separator, that in the United States is the comma, but,
# for instance, in France is the space and in Italy and in Germany is the dot
# dSep is the decimal separator, that in the United States is the dot, but in
# the other above mentioned countries is the comma. dSep has "None" as default
# value since integers haven't a decimal part.
def Split_Thousands(s, tSep=',', dSep=None):
    if s == None:
        return 0
    if not isinstance(s, str):
        s = str(s)
    if s.rfind('.')>0:
        rhs=s[s.rfind('.')+1:]
        s=s[:s.rfind('.')]
        if len(s) <= 3: return s + dSep + rhs
        return Split_Thousands(s[:-3], tSep) + tSep + s[-3:] + dSep + rhs
    else:
        if len(s) <= 3: return s
        return Split_Thousands(s[:-3], tSep) + tSep + s[-3:]


# Capitalize first letter in each word of string, returning string
def Capitalize_Words(stir):
    cstir = []
    ln = len(stir)
    wd = True
    for i in range(0,ln):
        let = stir[i]
        if let in string.ascii_letters:
            if wd: cstir.append(let.upper())
            else:  cstir.append(let)
            wd = False
        else:
            wd = True
            cstir.append(let)
    return List_to_String(cstir)


# Returns a string made of num copies of input string.
def String_Fill(stir, num):
    s = ''
    assert(num >= 0)
    s = stir * num
    return s


# Transforms the string for into a new string for purposes of comparison
# Skip first skip_fields and skip first skip_chars.
# Returns the transformed string
def Transform_Skip(stir, skip_fields, skip_chars):
    start = 0
    inlen = len(stir)
    if skip_fields > 0:			# Skip num_minus fields.
        infield = False
        while start < inlen and skip_fields > 0:
            if stir[start] in [' ', '\t']:
                if infield:
                    skip_fields -= 1
                    infield = False
            else:
                infield = True
            start += 1
        if skip_fields == 0 and start < inlen:
            start -= 1

    start += skip_chars			# Skip skip_chars spaces.

    return(stir[start:])


# Transforms the string for into a new string for purposes of comparison
# if kill_chars string is defined, remove each kill_char from str
# Returns the transformed string
def Transform_Kill(stir, kill_chars=None):
    lis = []				# Cheap to append to a list

    if kill_chars:
        for s in stir:			# Iterate over stirng
            ok = True
            for c in kill_chars:	# Iterate over kill_chars
                if s == c:
                    ok = False
                    break
            if ok: lis.append(s)
        stir = List_to_String(lis)

    return stir


# Transforms the string for into a new string for purposes of comparison
# Returns the transformed string
def Transform_Attribute(stir):
    att      = '\t\t-- '

    i = stir.rfind(att)
    if i != -1: stir = stir[:i]		# Remove atrribute
    return stir


# Convert string of characters into list of word strings
def To_Alpha(stir):
    word = ''
    liszt = []

    for c in stir:
        if c.isalpha():
            word += c
        else:
            if len(word) > 0:
                liszt.append(word)
            word = ''
    if word != '':
        liszt.append(word)
    return liszt


# Returns true if vowel present, else false.
def Has_Vowel(stir):
    for ch in stir:
        if Is_Vowel(ch):
            return True
    return False


# Returns true if all digits, else false.
def All_Digits(stir):
    for ch in stir:
        if not ch.isdigit(): return False
    return True


# Returns true if q is followed by u, else false.
def Q_Test(stir):

    leng = len(stir)
    lstr = stir.lower()
    for i in range (0, leng-1):
        if lstr[i] == 'q' and lstr[i + 1] != 'u':
            return False

    if lstr[leng - 1] == 'q':
        return False
    return True


# Returns true if no more than 4 vowels are in a row.
def V_Len_Test(stir):
    j = 0
    maxj = 0

    leng = len(stir)
    for i in range (0, leng):
        if Is_Vowel(stir[i]):
            j += 1
            if j > maxj:
                maxj = j
        else:
            j = 0
    return maxj <= 4


# Returns true if no more than 6 consonants are in a row.
def C_Len_Test(stir):
    j = 0
    maxj = 0

    leng = len(stir)
    for i in range (0, leng):
        if Is_Vowel(stir[i]):
            j = 0
        else:
            j += 1
            if j > maxj:
                maxj = j
    return maxj <= 6


# Remove trailing blanks from str, return processed str
# Blanks include space, tab, cr, lf, ff, bs
def Str_Tblank(stir):
    ln = len(stir)
    mn = ln

    # This goes from 0..ln-1 backwards
    for i in range(ln-1, -1, -1):
        if Is_Blank(stir[i]):
            mn = i
        else:
            break
    return stir[0:mn]


# Convert a string with spaces to have tabs when appropriate
# There are probably more elegant ways to do this...
def Spaces_To_Tabs(stir, all, tabstop):
    ln = len(stir)
    pos = 0
    vpos = 0
    quo_str = False
    tab = '\t'
    sp  = ' '

    while pos < ln:
        # quo_str is true when in quoted string.
        if all == False and (stir[pos] == "'" or stir[pos] == '"'):
            quo_str = not quo_str

        if stir[pos] == tab:
            vpos += tabstop - (vpos % tabstop)
        else:
            vpos += 1

        if quo_str == False:
            if (vpos % tabstop) == 0 and vpos > 0:	# We are at a tab-stop.
                # If a tab and a space before it, do some processing.
                if (stir[pos] == tab or stir[pos] == sp) and stir[pos-1] == sp:
                    i = pos
                    while (i > 0):
                        if not(stir[i-1] == sp and i + tabstop-1 > pos): break
                        i -= 1
                    # Now i is set to the first space that we remove.
                    j = pos - i
                    stir = stir[:i] + tab + stir[pos+1:]
                    ln -= j
                    pos -= j
        pos += 1
    return(stir)


# Process command line number and possible letter suffix
# possible letters multiply the number, typically by 1024^n or 1000^n
def Command_Num_Process(stir):

    neg = False			# Deal with negative numbers
    if stir[0] == '-':
        neg = True
        stir = stir[1:]

    number = ''			# Break str into number and the rest
    rest = ''
    for i in range(0, len(stir)):
        if stir[i].isdigit():
            number += stir[i]
        else:
            rest = stir[i:]
            break

    try:
        n = int(number)
    except:
        jefflib.Print_Error(stir, "isn't an integer")

    if   rest == 'b':  n *= 512
    elif rest == 'kB': n *= 1000
    elif rest == 'K':  n *= 1024
    elif rest == 'MB': n *= 1000 * 1000
    elif rest == 'M':  n *= 1024 * 1024
    elif rest == 'GB': n *= 1000 * 1000 * 1000
    elif rest == 'G':  n *= 1024 * 1024 * 1024
    elif rest == '': pass
    else:
        jefflib.Print_Error('suffix', rest, 'not recognized')
    if neg == True: n *= -1
    return n


# Returns true if valid two letter combos, else false.
def Two_Test(stir):
    two_letter = [
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1],
[1,1,1,1,1,1,0,1,1,1,0,1,1,1,1,1,0,1,1,1,1,1,1,0,1,0],
[1,0,1,1,1,1,0,1,1,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1],
[1,1,1,1,1,1,0,1,1,1,0,1,1,1,1,0,0,1,1,1,1,0,1,0,1,0],
[1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,0,1,0,1,1],
[1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1,1,0,1,0,1,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1],
[1,0,0,0,1,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,0,1,0,1,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1],
[1,1,1,1,1,1,0,1,1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,0,1,0,1,0],
[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,0],
[1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1],
[1,0,0,0,1,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0,1,1,0,0,1,1],
[1,1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1,1,1,0,1,0,1,1],
[1,0,1,0,1,1,1,1,1,0,0,1,0,0,1,1,1,0,1,1,1,0,1,0,1,0],
[1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,1,1,0,1,1,0,1],
[1,0,0,0,1,0,0,0,1,0,0,1,0,0,1,0,0,0,0,0,1,1,1,0,1,1], ]

    leng = len(stir)
    lstr = stir.lower()
    for i in range (0, leng-1):
        assert lstr[i].isalpha()
        assert lstr[i+1].isalpha()
        if two_letter[ord(lstr[i])-97][ord(lstr[i+1])-97] == 0:
            return False
    return True


# Returns true if starting 2 letters ok, else false.
def Start_2_ok(stir):
    sta = [
"aa","ab","ac","ad","ae","af","ag","ah","ai","aj","ak","al","am","an","ao",
"ap","aq","ar","as","at","au","av","aw","ax","ay","az","ba","be","bh","bi",
"bl","bm","bo","br","bs","bt","bu","by","ca","cb","cd","ce","ch","ci","cl",
"co","cp","cr","cu","cy","cz","da","de","dh","di","dj","dn","do","dr","du",
"dw","dy","ea","eb","ec","ed","ee","ef","eg","eh","ei","ej","ek","el","em",
"en","eo","ep","eq","er","es","et","eu","ev","ew","ex","ey","ez","fa","fb",
"fc","fd","fe","fi","fj","fl","fm","fo","fp","fr","ft","fu","ga","ge","gh",
"gi","gl","gm","gn","go","gp","gr","gs","gu","gw","gy","ha","he","hi","ho",
"hr","hu","hy","ia","ib","ic","id","ie","if","ig","ii","ik","il","im","in",
"io","ip","ir","is","it","iv","iz","ja","je","ji","jo","ju","ka","ke","kh",
"ki","kl","kn","ko","kr","ku","kw","ky","la","le","li","ll","lo","ls","lt",
"lu","ly","ma","mb","mc","me","mi","mn","mo","mp","mr","mu","my","na","nb",
"nc","nd","ne","ng","ni","no","nr","ns","nt","nu","ny","oa","ob","oc","od",
"oe","of","og","oh","oi","ok","ol","om","on","oo","op","or","os","ot","ou",
"ov","ow","ox","oy","oz","pa","pb","pe","pf","ph","pi","pl","pn","po","pp",
"pr","ps","pt","pu","pv","py","qa","qe","qu","ra","rc","re","rh","ri","ro",
"rp","rs","ru","rw","ry","sa","sc","se","sf","sh","si","sk","sl","sm","sn",
"so","sp","sq","sr","ss","st","su","sv","sw","sy","ta","te","th","ti","tn",
"to","tr","ts","tt","tu","tv","tw","ty","ub","uc","ug","uk","ul","um","un",
"up","ur","us","ut","va","ve","vi","vl","vo","vr","vu","vy","wa","we","wh",
"wi","wo","wr","wu","wy","xa","xe","xy","ya","ye","yi","ym","yo","yp","yt",
"yu","yv","yw","za","ze","zi","zl","zo","zu" ]

    lstr = stir.lower()
    if len(lstr) < 2:
        return False
    i = bisect.bisect_left(sta, lstr[0:2])
    if sta[i] == lstr[0:2]:
        return True
    return False


# Returns true if finishing 2 letters ok, else false.
def Finish_2_ok(stir):
    fin = [
"aa","ab","ac","ad","ae","af","ag","ah","ai","aj","ak","al","am","an","ao",
"ap","aq","ar","as","at","au","av","aw","ax","ay","az","ba","bb","bc","be",
"bi","bm","bn","bo","bs","bt","bu","by","ca","cc","ce","ch","ci","ck","cm",
"co","cp","cr","cs","ct","cy","da","dc","dd","de","dh","di","dn","do","ds",
"dt","du","dy","ea","eb","ec","ed","ee","ef","eg","eh","ei","ek","el","em",
"en","eo","ep","er","es","et","eu","ev","ew","ex","ey","ez","fa","fe","ff",
"fo","ft","fu","fy","ga","ge","gg","gh","gi","gm","gn","go","gs","gy","ha",
"hd","he","hi","hl","hm","hn","ho","hr","hs","ht","hu","hy","ia","ib","ic",
"id","ie","if","ig","ih","ii","ik","il","im","in","io","ip","ir","is","it",
"iv","ix","iz","ja","ji","jo","ju","ka","ke","ki","ko","ks","ku","ky","la",
"lb","lc","ld","le","lf","li","lk","ll","lm","ln","lo","lp","ls","lt","lu",
"ly","lz","ma","mb","mc","me","mh","mi","mm","mn","mo","mp","ms","mt","mw",
"my","na","nc","nd","ne","ng","nh","ni","nk","nn","no","np","ns","nt","nu",
"nx","ny","nz","oa","ob","oc","od","oe","of","og","oh","oi","ok","ol","om",
"on","oo","op","or","os","ot","ou","ov","ow","ox","oy","oz","pa","pc","pe",
"ph","pi","pm","po","pp","ps","pt","pu","py","ra","rb","rc","rd","re","rf",
"rg","rh","ri","rk","rl","rm","rn","ro","rp","rr","rs","rt","ru","rw","rx",
"ry","sa","sc","se","sf","sh","si","sk","sm","sn","so","sp","sr","ss","st",
"su","sy","ta","tc","td","te","th","ti","tj","tl","tm","to","ts","tt","tu",
"tv","ty","tz","ua","ub","ud","ue","uf","ug","uh","ui","uk","ul","um","un",
"uo","up","ur","us","ut","ux","uy","uz","va","vc","ve","vi","vo","vp","vu",
"vy","wa","wd","we","wi","wk","wl","wn","wo","wr","ws","wt","wx","wy","xe",
"xi","xt","xy","ya","yc","yd","ye","yl","ym","yn","yo","yp","yr","ys","yt",
"yu","yx","za","ze","zi","zo","zu","zy","zz" ]

    lstr = stir.lower()
    ln = len(lstr)
    if ln < 2:
        return False
    i = bisect.bisect_left(fin, lstr[ln-2:])
    if fin[i] == lstr[ln-2:]:
        return True
    return False


# Returns true if starting 3 letters ok, else false.
def Start_3_ok(stir):
    sta = [
"aar", "aba", "abb", "abc", "abd", "abe", "abh", "abi", "abj", "abl",
"abn", "abo", "abr", "abs", "abu", "aby", "aca", "acc", "ace", "ach",
"aci", "ack", "acm", "aco", "acq", "acr", "act", "acu", "acy", "ada",
"add", "ade", "adh", "adi", "adj", "adk", "adl", "adm", "ado", "adr",
"ads", "adu", "adv", "aeg", "aen", "aeo", "aer", "aes", "afa", "aff",
"afg", "afi", "afl", "afo", "afr", "aft", "aga", "age", "agg", "agh",
"agi", "agl", "agn", "ago", "agr", "agu", "agw", "ahe", "ahm", "aho",
"aid", "aik", "ail", "aim", "ain", "air", "ais", "ait", "aja", "ake",
"aki", "akr", "ala", "alb", "alc", "ald", "ale", "alf", "alg", "alh",
"ali", "alk", "all", "alm", "alo", "alp", "alr", "als", "alt", "alu",
"alv", "alw", "aly", "ama", "amb", "ame", "amh", "ami", "amm", "amo",
"amp", "amr", "ams", "amt", "amu", "amy", "ana", "anc", "and", "ane",
"ang", "anh", "ani", "ank", "ann", "ano", "ans", "ant", "anu", "anv",
"anx", "any", "aor", "apa", "ape", "aph", "api", "apl", "apo", "app",
"apr", "aps", "apt", "aqu", "ara", "arb", "arc", "ard", "are", "arg",
"arh", "ari", "ark", "arl", "arm", "arn", "aro", "arp", "arr", "ars",
"art", "aru", "ary", "asb", "asc", "ase", "ash", "asi", "ask", "asl",
"aso", "asp", "ass", "ast", "asu", "asy", "ata", "atc", "ate", "ath",
"atk", "atl", "atm", "ato", "atr", "att", "atw", "aty", "aub", "auc",
"aud", "aue", "aug", "auk", "aun", "aur", "aus", "aut", "aux", "ava",
"ave", "avi", "avo", "awa", "awe", "awf", "awh", "awk", "awl", "awn",
"awo", "awr", "axe", "axi", "axl", "axo", "aye", "ayl", "aza", "aze",
"azi", "azt", "azu", "bab", "bac", "bad", "baf", "bag", "bah", "bai",
"bak", "bal", "bam", "ban", "bap", "bar", "bas", "bat", "bau", "bav",
"baw", "bax", "bay", "baz", "bea", "beb", "bec", "bed", "bee", "bef",
"beg", "beh", "bei", "bel", "bem", "ben", "beo", "bep", "beq", "ber",
"bes", "bet", "bev", "bew", "bey", "bez", "bho", "bhu", "bia", "bib",
"bic", "bid", "bie", "bif", "big", "bih", "bij", "bik", "bil", "bim",
"bin", "bio", "bip", "bir", "bis", "bit", "biv", "biz", "bla", "ble",
"bli", "blo", "blu", "bly", "bmw", "boa", "bob", "boc", "bod", "boe",
"bog", "boh", "boi", "bol", "bom", "bon", "boo", "bop", "bor", "bos",
"bot", "bou", "bov", "bow", "box", "boy", "bra", "bre", "bri", "bro",
"bru", "bry", "bst", "btl", "bub", "buc", "bud", "bue", "buf", "bug",
"bui", "buj", "bul", "bum", "bun", "buo", "bur", "bus", "but", "bux",
"buy", "buz", "bye", "byg", "byl", "byp", "byr", "bys", "byt", "byw",
"byz", "cab", "cac", "cad", "cae", "caf", "cag", "cah", "cai", "caj",
"cak", "cal", "cam", "can", "cap", "car", "cas", "cat", "cau", "cav",
"caw", "cay", "cbs", "cdc", "cea", "cec", "ced", "cei", "cel", "cem",
"cen", "cep", "cer", "ces", "cet", "cey", "cez", "cha", "che", "chi",
"chl", "cho", "chr", "chu", "cia", "cic", "cid", "cig", "cil", "cim",
"cin", "cip", "cir", "cis", "cit", "civ", "cla", "cle", "cli", "clo",
"clu", "cly", "coa", "cob", "coc", "cod", "coe", "cof", "cog", "coh",
"coi", "cok", "col", "com", "con", "coo", "cop", "coq", "cor", "cos",
"cot", "cou", "cov", "cow", "cox", "coy", "coz", "cpa", "cra", "cre",
"cri", "cro", "crt", "cru", "cry", "cub", "cuc", "cud", "cue", "cuf",
"cui", "cul", "cum", "cun", "cup", "cur", "cus", "cut", "cya", "cyb",
"cyc", "cyg", "cyl", "cyn", "cyp", "cyr", "cys", "cyt", "cza", "cze",
"dab", "dac", "dad", "dae", "daf", "dag", "dah", "dai", "dak", "dal",
"dam", "dan", "dap", "dar", "das", "dat", "dau", "dav", "daw", "day",
"daz", "dea", "deb", "dec", "ded", "dee", "def", "deg", "deh", "dei",
"dej", "del", "dem", "den", "deo", "dep", "der", "des", "det", "deu",
"dev", "dew", "dex", "dey", "dha", "dia", "dib", "dic", "did", "die",
"dif", "dig", "dih", "dil", "dim", "din", "dio", "dip", "dir", "dis",
"dit", "diu", "div", "dix", "diz", "dja", "dna", "dni", "dob", "doc",
"dod", "doe", "dof", "dog", "doh", "dol", "dom", "don", "doo", "dop",
"dor", "dos", "dot", "dou", "dov", "dow", "doy", "doz", "dra", "dre",
"dri", "dro", "dru", "dry", "dua", "dub", "duc", "dud", "due", "duf",
"dug", "duk", "dul", "dum", "dun", "duo", "dup", "dur", "dus", "dut",
"dwa", "dwe", "dwi", "dwy", "dya", "dye", "dyi", "dyk", "dyl", "dyn",
"dys", "eac", "eag", "ear", "eas", "eat", "eav", "ebb", "ebe", "ebo",
"ebu", "ecc", "ech", "ecl", "eco", "ecs", "ecu", "edd", "ede", "edg",
"edi", "edm", "edn", "edt", "edu", "edw", "eel", "eeo", "eer", "eff",
"eft", "ega", "egg", "ego", "egr", "egy", "ehr", "eid", "eig", "eil",
"ein", "eir", "eis", "eit", "eja", "eje", "eke", "eks", "ekt", "ela",
"elb", "eld", "ele", "elf", "elg", "eli", "elk", "ell", "elm", "elo",
"els", "elt", "elu", "elv", "ely", "ema", "emb", "emc", "eme", "emi",
"emm", "emo", "emp", "emu", "ena", "enc", "end", "ene", "enf", "eng",
"enh", "eni", "enj", "enl", "enm", "eno", "enq", "enr", "ens", "ent",
"enu", "env", "enz", "eoc", "eoh", "eos", "epa", "eph", "epi", "epo",
"eps", "equ", "era", "erb", "erd", "ere", "erg", "eri", "erl", "ern",
"ero", "err", "ers", "eru", "erv", "erw", "esc", "esk", "esm", "eso",
"esp", "esq", "ess", "est", "eta", "etc", "ete", "eth", "eti", "etr",
"etu", "ety", "euc", "eug", "eul", "eum", "eun", "eup", "eur", "eut",
"eva", "eve", "evi", "evo", "evz", "ewe", "ewi", "exa", "exc", "exe",
"exh", "exi", "exo", "exp", "exq", "ext", "exu", "exx", "eye", "eze",
"ezr", "faa", "fab", "fac", "fad", "fae", "faf", "fag", "fah", "fai",
"fak", "fal", "fam", "fan", "far", "fas", "fat", "fau", "faw", "fay",
"faz", "fbi", "fcc", "fda", "fea", "feb", "fec", "fed", "fee", "fei",
"fel", "fem", "fen", "fer", "fes", "fet", "feu", "fev", "few", "fia",
"fib", "fic", "fid", "fie", "fif", "fig", "fil", "fin", "fir", "fis",
"fit", "fiv", "fix", "fiz", "fjo", "fla", "fle", "fli", "flo", "flu",
"fly", "fmc", "foa", "fob", "foc", "fod", "foe", "fog", "foi", "fol",
"fom", "fon", "foo", "fop", "for", "fos", "fou", "fov", "fow", "fox",
"foy", "fpc", "fra", "fre", "fri", "fro", "fru", "fry", "ftc", "fuc",
"fud", "fue", "fug", "fuj", "ful", "fum", "fun", "fur", "fus", "fut",
"fuz", "gab", "gad", "gae", "gaf", "gag", "gai", "gal", "gam", "gan",
"gao", "gap", "gar", "gas", "gat", "gau", "gav", "gaw", "gay", "gaz",
"gea", "gec", "gee", "geg", "gei", "gel", "gem", "gen", "geo", "ger",
"ges", "get", "gey", "gha", "ghe", "gho", "gia", "gib", "gid", "gif",
"gig", "gil", "gim", "gin", "gio", "gir", "gis", "giu", "giv", "gla",
"gle", "gli", "glo", "glu", "gly", "gmt", "gna", "gne", "gno", "gnp",
"gnu", "goa", "gob", "god", "goe", "gof", "gog", "gol", "gon", "goo",
"gop", "gor", "gos", "got", "gou", "gov", "gow", "gpo", "gra", "gre",
"gri", "gro", "gru", "gry", "gsa", "gua", "gub", "gue", "guf", "gug",
"gui", "gul", "gum", "gun", "gur", "gus", "gut", "guy", "guz", "gwe",
"gwy", "gym", "gyp", "gyr", "haa", "hab", "hac", "had", "haf", "hag",
"hah", "hai", "hal", "ham", "han", "hap", "har", "has", "hat", "hau",
"hav", "haw", "hay", "haz", "hea", "heb", "hec", "hed", "hee", "hef",
"heg", "hei", "hel", "hem", "hen", "hep", "her", "hes", "het", "heu",
"hew", "hex", "hey", "hia", "hib", "hic", "hid", "hie", "hif", "hig",
"hij", "hik", "hil", "him", "hin", "hip", "hir", "his", "hit", "hiv",
"hoa", "hob", "hoc", "hod", "hoe", "hof", "hog", "hoi", "hok", "hol",
"hom", "hon", "hoo", "hop", "hor", "hos", "hot", "hou", "hov", "how",
"hoy", "hro", "hub", "huc", "hud", "hue", "huf", "hug", "huh", "hul",
"hum", "hun", "hur", "hus", "hut", "hux", "huz", "hya", "hyb", "hyd",
"hye", "hyg", "hyi", "hym", "hyp", "hys", "iam", "ibe", "ibi", "ibm",
"ibn", "ica", "icc", "ice", "ich", "ici", "ico", "icy", "ida", "ide",
"idi", "idl", "ido", "idy", "iee", "iff", "ifn", "igl", "ign", "iii",
"ike", "ile", "ili", "ill", "ilo", "ily", "ima", "imb", "imi", "imm",
"imp", "ina", "inb", "inc", "ind", "ine", "inf", "ing", "inh", "ini",
"inj", "ink", "inl", "inm", "inn", "ino", "inp", "inq", "inr", "ins",
"int", "inu", "inv", "inw", "iod", "ion", "iot", "iow", "ipe", "ips",
"ira", "ire", "iri", "irk", "irm", "iro", "irr", "irs", "irv", "irw",
"isa", "ise", "isf", "isi", "isl", "isn", "iso", "isr", "iss", "ist",
"ita", "itc", "ite", "ith", "iti", "ito", "its", "itt", "iva", "ive",
"ivo", "ivy", "izv", "jab", "jac", "jad", "jae", "jag", "jai", "jak",
"jal", "jam", "jan", "jap", "jar", "jas", "jau", "jav", "jaw", "jay",
"jaz", "jea", "jed", "jee", "jef", "jeh", "jej", "jel", "jen", "jeo",
"jer", "jes", "jet", "jew", "jib", "jif", "jig", "jil", "jim", "jin",
"jit", "jiv", "joa", "job", "joc", "joe", "jog", "joh", "joi", "jok",
"jol", "jon", "jor", "jos", "jot", "jou", "jov", "jow", "joy", "jua",
"jub", "jud", "jug", "jui", "juj", "juk", "jul", "jum", "jun", "jup",
"jur", "jus", "jut", "juv", "jux", "kab", "kad", "kaf", "kah", "kai",
"kaj", "kal", "kam", "kan", "kao", "kap", "kar", "kas", "kat", "kau",
"kav", "kay", "kaz", "kea", "ked", "kee", "keg", "kei", "kel", "kem",
"ken", "kep", "ker", "kes", "ket", "kev", "key", "kha", "khm", "khr",
"kib", "kic", "kid", "kie", "kig", "kik", "kil", "kim", "kin", "kio",
"kip", "kir", "kis", "kit", "kiv", "kiw", "kla", "kle", "kli", "klu",
"kly", "kna", "kne", "kni", "kno", "knu", "koa", "koc", "kod", "koe",
"koh", "koi", "kol", "kom", "kon", "kop", "kor", "kos", "kow", "kra",
"kre", "kri", "kro", "kru", "kry", "kud", "kuh", "kul", "kum", "kur",
"kuw", "kwa", "kyl", "kyo", "lab", "lac", "lad", "laf", "lag", "lah",
"lai", "lak", "lam", "lan", "lao", "lap", "lar", "las", "lat", "lau",
"lav", "law", "lax", "lay", "laz", "lea", "leb", "lec", "led", "lee",
"lef", "leg", "leh", "lei", "lel", "lem", "len", "leo", "lep", "ler",
"les", "let", "leu", "lev", "lew", "lex", "ley", "lia", "lib", "lic",
"lid", "lie", "lif", "lig", "lik", "lil", "lim", "lin", "lio", "lip",
"liq", "lis", "lit", "liv", "liz", "llo", "loa", "lob", "loc", "lod",
"loe", "lof", "log", "loi", "lok", "lol", "lom", "lon", "loo", "lop",
"loq", "lor", "los", "lot", "lou", "lov", "low", "loy", "loz", "lsi",
"ltv", "lub", "luc", "lud", "luf", "lug", "lui", "luk", "lul", "lum",
"lun", "lup", "lur", "lus", "lut", "lux", "luz", "lyc", "lyd", "lye",
"lyi", "lyk", "lyl", "lym", "lyn", "lyo", "lyr", "lys", "mab", "mac",
"mad", "mae", "mag", "mah", "mai", "maj", "mak", "mal", "mam", "man",
"mao", "map", "mar", "mas", "mat", "mau", "mav", "maw", "max", "may",
"maz", "mba", "mca", "mcb", "mcc", "mcd", "mce", "mcf", "mcg", "mch",
"mci", "mck", "mcl", "mcm", "mcn", "mcp", "mea", "mec", "med", "mee",
"meg", "mei", "mek", "mel", "mem", "men", "mep", "mer", "mes", "met",
"mew", "mex", "mey", "mez", "mia", "mic", "mid", "mie", "mif", "mig",
"mik", "mil", "mim", "min", "mio", "mir", "mis", "mit", "mix", "miz",
"mne", "moa", "mob", "moc", "mod", "moe", "mog", "moh", "moi", "mol",
"mom", "mon", "moo", "mop", "mor", "mos", "mot", "mou", "mov", "mow",
"moy", "moz", "mph", "mrs", "muc", "mud", "mue", "muf", "mug", "mui",
"muk", "mul", "mum", "mun", "muo", "mur", "mus", "mut", "muz", "myc",
"mye", "myl", "myn", "myo", "myr", "mys", "myt", "naa", "nab", "nad",
"nag", "nai", "nak", "nam", "nan", "nao", "nap", "nar", "nas", "nat",
"nau", "nav", "nay", "naz", "nbc", "nbs", "nca", "ncr", "ndj", "nea",
"neb", "nec", "ned", "nee", "nef", "neg", "neh", "nei", "nel", "nem",
"neo", "nep", "ner", "nes", "net", "neu", "nev", "new", "nex", "ngu",
"nia", "nib", "nic", "nie", "nig", "nih", "nik", "nil", "nim", "nin",
"nio", "nip", "nir", "nit", "nix", "noa", "nob", "noc", "nod", "noe",
"noi", "nol", "nom", "non", "noo", "nor", "nos", "not", "nou", "nov",
"now", "nox", "noz", "nrc", "nsf", "nti", "nua", "nub", "nuc", "nud",
"nug", "nui", "nul", "num", "nun", "nup", "nur", "nut", "nuz", "nyc",
"nyl", "nym", "nyq", "nyu", "oaf", "oak", "oar", "oas", "oat", "obd",
"obe", "obf", "obi", "obj", "obl", "obn", "obo", "obs", "obt", "obv",
"oca", "occ", "oce", "oct", "ocu", "odd", "ode", "odi", "odo", "ody",
"oed", "oer", "off", "oft", "ogd", "ogl", "ogr", "ohi", "ohm", "oil",
"oin", "oka", "oki", "okl", "ola", "old", "ole", "olg", "oli", "ols",
"oly", "oma", "ome", "omi", "omn", "onc", "one", "ong", "oni", "onl",
"ono", "onr", "ons", "ont", "onu", "onw", "ony", "ood", "ooz", "opa",
"ope", "oph", "opi", "opo", "opp", "opt", "opu", "ora", "orb", "orc",
"ord", "ore", "org", "ori", "ork", "orl", "orn", "oro", "orp", "orr",
"ort", "orv", "orw", "osa", "osb", "osc", "osg", "osh", "osi", "osl",
"osm", "osp", "oss", "ost", "osw", "oth", "oti", "ott", "oua", "ouc",
"oug", "oun", "our", "ous", "out", "ouz", "ova", "ove", "ovi", "owe",
"owi", "owl", "own", "oxa", "oxc", "oxe", "oxf", "oxi", "oxn", "oxy",
"oys", "oza", "ozo", "pab", "pac", "pad", "pae", "pag", "pai", "pak",
"pal", "pam", "pan", "pao", "pap", "par", "pas", "pat", "pau", "pav",
"paw", "pax", "pay", "paz", "pbs", "pea", "peb", "pec", "ped", "pee",
"peg", "pej", "pek", "pel", "pem", "pen", "peo", "pep", "per", "pes",
"pet", "peu", "pew", "pfe", "pfi", "pha", "phd", "phe", "phi", "phl",
"pho", "phr", "pht", "phy", "pia", "pic", "pid", "pie", "pig", "pik",
"pil", "pim", "pin", "pio", "pip", "piq", "pir", "pis", "pit", "piu",
"piv", "pix", "piz", "pla", "ple", "pli", "plo", "plu", "ply", "pne",
"poa", "poc", "pod", "poe", "pog", "poi", "pok", "pol", "pom", "pon",
"poo", "pop", "por", "pos", "pot", "pou", "pov", "pow", "poy", "ppm",
"pra", "pre", "pri", "pro", "pru", "pry", "psa", "pse", "psi", "psy",
"pta", "pto", "pub", "puc", "pud", "pue", "puf", "pug", "pui", "puk",
"pul", "pum", "pun", "pup", "pur", "pus", "put", "puz", "pvc", "pyg",
"pyh", "pyk", "pyl", "pyo", "pyr", "pyt", "qat", "qed", "qua", "que",
"qui", "quo", "rab", "rac", "rad", "rae", "raf", "rag", "rai", "raj",
"rak", "ral", "ram", "ran", "rao", "rap", "rar", "ras", "rat", "rau",
"rav", "raw", "ray", "raz", "rca", "rea", "reb", "rec", "red", "ree",
"ref", "reg", "reh", "rei", "rej", "rel", "rem", "ren", "rep", "req",
"rer", "res", "ret", "reu", "rev", "rew", "rex", "rey", "rha", "rhe",
"rhi", "rho", "rhu", "rhy", "rib", "ric", "rid", "rie", "rif", "rig",
"ril", "rim", "rin", "rio", "rip", "ris", "rit", "riv", "riy", "roa",
"rob", "roc", "rod", "roe", "rog", "roi", "rol", "rom", "ron", "roo",
"rop", "ros", "rot", "rou", "rov", "row", "roy", "rpm", "rsv", "rua",
"rub", "ruc", "rud", "rue", "ruf", "rug", "rui", "rul", "rum", "run",
"rup", "rur", "rus", "rut", "rwa", "rya", "ryd", "rye", "sab", "sac",
"sad", "saf", "sag", "sah", "sai", "sak", "sal", "sam", "san", "sao",
"sap", "sar", "sas", "sat", "sau", "sav", "saw", "sax", "say", "sca",
"sce", "sch", "sci", "scl", "scm", "sco", "scr", "scu", "scy", "sea",
"seb", "sec", "sed", "see", "seg", "sei", "sel", "sem", "sen", "seo",
"sep", "seq", "ser", "ses", "set", "sev", "sew", "sex", "sey", "sfo",
"sha", "she", "shi", "sho", "shr", "shu", "shy", "sia", "sib", "sic",
"sid", "sie", "sif", "sig", "sik", "sil", "sim", "sin", "sio", "sip",
"sir", "sis", "sit", "siv", "six", "siz", "ska", "ske", "ski", "sku",
"sky", "sla", "sle", "sli", "slo", "slu", "sly", "sma", "sme", "smi",
"smo", "smu", "smy", "sna", "sne", "sni", "sno", "snu", "sny", "soa",
"sob", "soc", "sod", "sof", "sog", "soi", "soj", "sol", "som", "son",
"soo", "sop", "sor", "sou", "sov", "sow", "soy", "spa", "spe", "sph",
"spi", "spl", "spo", "spr", "spu", "spy", "squ", "sri", "sst", "sta",
"ste", "sti", "sto", "str", "stu", "sty", "sua", "sub", "suc", "sud",
"sue", "suf", "sug", "sui", "sul", "sum", "sun", "sup", "sur", "sus",
"sut", "suz", "sve", "swa", "swe", "swi", "swo", "swu", "syb", "syc",
"syd", "sye", "syk", "syl", "sym", "syn", "syr", "sys", "tab", "tac",
"tad", "taf", "tag", "tah", "tai", "tak", "tal", "tam", "tan", "tao",
"tap", "tar", "tas", "tat", "tau", "tav", "taw", "tax", "tay", "tea",
"tec", "ted", "tee", "tef", "teg", "teh", "tek", "tel", "tem", "ten",
"tep", "ter", "tes", "tet", "teu", "tex", "tha", "the", "thi", "tho",
"thr", "thu", "thw", "thy", "tib", "tic", "tid", "tie", "tif", "tig",
"til", "tim", "tin", "tio", "tip", "tir", "tis", "tit", "tnt", "toa",
"tob", "toc", "tod", "toe", "tof", "tog", "toi", "tok", "tol", "tom",
"ton", "too", "top", "tor", "tos", "tot", "tou", "tow", "tox", "toy",
"tra", "tre", "tri", "tro", "tru", "trw", "try", "tsa", "tsu", "ttl",
"tty", "tub", "tuc", "tud", "tue", "tuf", "tug", "tui", "tul", "tum",
"tun", "tup", "tur", "tus", "tut", "tux", "tva", "twa", "twe", "twi",
"two", "twx", "tyb", "tyc", "tyi", "tyl", "typ", "tyr", "tys", "ubi",
"ucl", "uga", "ugh", "ugl", "ukr", "ula", "ulc", "ull", "uls", "ult",
"uly", "umb", "ump", "una", "unb", "unc", "und", "une", "uni", "unk",
"unr", "unt", "unw", "upb", "upc", "upd", "upe", "upg", "uph", "upk",
"upl", "upo", "upp", "upr", "ups", "upt", "upw", "ura", "urb", "urc",
"ure", "urg", "uri", "urn", "urs", "uru", "usa", "usc", "usd", "use",
"usg", "ush", "usi", "usn", "usp", "uss", "usu", "uta", "ute", "uti",
"utm", "uto", "utt", "vac", "vad", "vag", "vai", "val", "vam", "van",
"var", "vas", "vat", "vau", "vea", "vec", "ved", "vee", "veg", "veh",
"vei", "vel", "ven", "ver", "ves", "vet", "vex", "via", "vib", "vic",
"vid", "vie", "vig", "vii", "vik", "vil", "vin", "vio", "vir", "vis",
"vit", "viv", "vix", "viz", "vla", "voc", "vog", "voi", "vol", "vom",
"von", "voo", "vor", "vos", "vot", "vou", "vow", "voy", "vre", "vul",
"vyi", "waa", "wab", "wac", "wad", "waf", "wag", "wah", "wai", "wak",
"wal", "wan", "wap", "war", "was", "wat", "wav", "wax", "way", "wea",
"web", "wec", "wed", "wee", "weh", "wei", "wel", "wen", "wep", "wer",
"wes", "wet", "wey", "wha", "whe", "whi", "who", "whu", "why", "wic",
"wid", "wie", "wif", "wig", "wil", "win", "wip", "wir", "wis", "wit",
"wiv", "wiz", "wob", "woe", "wok", "wol", "wom", "won", "woo", "wop",
"wor", "wot", "wou", "wov", "wow", "wra", "wre", "wri", "wro", "wry",
"wuh", "wya", "wye", "wyl", "wym", "wyn", "wyo", "xav", "xen", "xer",
"xyl", "yac", "yah", "yak", "yal", "yam", "yan", "yao", "yap", "yaq",
"yar", "yat", "yaw", "yea", "yel", "yem", "yen", "yeo", "yes", "yet",
"yid", "yie", "yin", "yip", "ymc", "yod", "yog", "yok", "yol", "yon",
"yor", "yos", "you", "yow", "yps", "ytt", "yuc", "yug", "yuh", "yuk",
"yul", "yve", "ywc", "zac", "zag", "zai", "zam", "zan", "zap", "zea",
"zeb", "zei", "zel", "zen", "zer", "zes", "zet", "zeu", "zie", "zig",
"zil", "zim", "zin", "zio", "zip", "zir", "zlo", "zod", "zoe", "zom",
"zon", "zoo", "zor", "zou", "zur" ]

    lstr = stir.lower()
    if len(lstr) < 3:
        return False
    i = bisect.bisect_left(sta, lstr[0:3])
    if sta[i] == lstr[0:3]:
        return True
    return False


# Returns true if finishing 2 letters ok, else false.
def Finish_3_ok(stir):
    fin = [
"aac", "aag", "aam", "aan", "aar", "aas", "aba", "abc", "abe", "abi",
"abo", "abu", "aby", "aca", "ace", "ach", "ack", "acm", "aco", "acp",
"act", "acy", "ada", "add", "ade", "adh", "adi", "ado", "ady", "ael",
"aer", "afe", "aff", "aft", "afu", "afy", "aga", "age", "agg", "agi",
"agm", "ago", "agy", "aha", "ahl", "ahn", "aho", "aic", "aid", "aig",
"aii", "ail", "aim", "ain", "air", "ais", "ait", "ajo", "aka", "ake",
"aki", "ako", "aku", "aky", "ala", "alb", "alc", "ald", "ale", "alf",
"ali", "alk", "all", "alm", "alo", "alp", "als", "alt", "aly", "ama",
"amb", "ame", "ami", "amn", "amo", "amp", "ams", "amt", "amy", "ana",
"anc", "and", "ane", "ang", "anh", "ani", "ank", "ann", "ano", "ans",
"ant", "anx", "any", "anz", "aos", "apa", "ape", "aph", "apo", "app",
"aps", "apt", "apy", "ara", "arb", "arc", "ard", "are", "arf", "ari",
"ark", "arl", "arm", "arn", "aro", "arp", "arr", "ars", "art", "arx",
"ary", "asa", "ase", "ash", "asi", "ask", "asm", "aso", "asp", "ass",
"ast", "asy", "ata", "ate", "ath", "ati", "ato", "ats", "att", "aty",
"atz", "aub", "aud", "aue", "auf", "aug", "auk", "aul", "aum", "aun",
"aup", "aur", "aus", "aut", "aux", "ava", "ave", "avo", "avy", "awa",
"awd", "awe", "awi", "awk", "awl", "awn", "awr", "axe", "axi", "axy",
"aya", "aye", "ayo", "ays", "aza", "aze", "azi", "azy", "azz", "bad",
"bae", "bag", "bah", "bal", "bam", "ban", "bao", "bar", "bas", "bat",
"bay", "bbe", "bbi", "bbs", "bby", "bda", "bec", "bed", "bee", "beg",
"bel", "ben", "ber", "bes", "bet", "bex", "bey", "bhe", "bia", "bib",
"bic", "bid", "bie", "big", "bil", "bim", "bin", "bis", "bit", "biz",
"ble", "blo", "bly", "bmw", "boa", "bob", "boe", "bog", "bol", "bon",
"boo", "bop", "bot", "bou", "bow", "box", "boy", "bra", "bre", "bro",
"bst", "btl", "bub", "bud", "bue", "bug", "bul", "bum", "bun", "bur",
"bus", "but", "buy", "bya", "bye", "byl", "caa", "cab", "cac", "cad",
"cal", "cam", "can", "cao", "cap", "car", "cas", "cat", "caw", "cay",
"cbs", "cca", "cci", "cco", "cdc", "cea", "ced", "cee", "cel", "cen",
"cep", "cer", "ces", "cet", "cey", "che", "chi", "cho", "chs", "cht",
"chy", "cia", "cic", "cid", "cie", "cil", "cin", "cio", "cis", "cit",
"cke", "cko", "cks", "cky", "cla", "cle", "cme", "coa", "cob", "cod",
"cog", "col", "com", "con", "coo", "cop", "cor", "cos", "cot", "cow",
"cox", "coy", "cpa", "cra", "cre", "cro", "crt", "cry", "cta", "cti",
"cto", "cts", "cua", "cub", "cud", "cue", "cum", "cuo", "cup", "cur",
"cus", "cut", "dab", "dad", "dah", "dak", "dal", "dam", "dan", "dap",
"dar", "das", "dau", "daw", "day", "dby", "dde", "ddy", "dea", "dec",
"ded", "dee", "del", "dem", "den", "deo", "der", "des", "det", "dew",
"dex", "dey", "dge", "dgy", "dha", "dia", "dic", "did", "die", "dig",
"dii", "dil", "dim", "din", "dio", "dip", "dir", "dis", "dit", "dix",
"dle", "dly", "dma", "dna", "dne", "dod", "doe", "dog", "dol", "dom",
"don", "doo", "dor", "dos", "dot", "dow", "dox", "dpa", "dra", "dre",
"dro", "dry", "dst", "dth", "dub", "dud", "due", "dug", "dum", "dun",
"dup", "dus", "duz", "dye", "dys", "dzu", "ead", "eaf", "eah", "eak",
"eal", "eam", "ean", "eap", "ear", "eas", "eat", "eau", "eba", "ebb",
"ebe", "ebo", "ebt", "eby", "eca", "ece", "ech", "eck", "eco", "ect",
"ecy", "eda", "ede", "edo", "eds", "edt", "edy", "eed", "eee", "eef",
"eek", "eel", "eem", "een", "eep", "eer", "eet", "eff", "eft", "efy",
"ega", "ege", "egg", "egm", "ego", "egy", "ehr", "eia", "eic", "eid",
"eig", "eii", "eik", "eil", "eim", "ein", "eir", "eis", "eit", "eja",
"eka", "eke", "eks", "eky", "ela", "eld", "ele", "elf", "eli", "elk",
"ell", "elm", "elo", "elp", "els", "elt", "ely", "ema", "eme", "emi",
"emn", "emo", "emp", "emy", "ena", "end", "ene", "eng", "enh", "enn",
"eno", "ens", "ent", "enu", "eny", "enz", "eoc", "eod", "eof", "eon",
"eor", "eot", "epa", "epe", "eph", "eps", "ept", "epy", "era", "erb",
"erd", "ere", "erg", "eri", "erk", "erm", "ern", "ero", "err", "ers",
"ert", "eru", "ery", "esa", "ese", "esh", "esk", "ess", "est", "esy",
"eta", "etc", "ete", "eth", "eto", "ets", "ett", "ety", "etz", "eub",
"eud", "eue", "eum", "eup", "eur", "eus", "eva", "eve", "evi", "evy",
"ewa", "ewd", "ewe", "ewn", "ews", "ewt", "ewy", "ext", "exy", "eya",
"eye", "eys", "eze", "ezy", "faa", "fab", "fad", "fag", "fak", "fal",
"fan", "far", "fat", "faw", "fax", "fay", "fbi", "fcc", "fda", "feb",
"fed", "fee", "fel", "fen", "fer", "fet", "few", "fey", "ffe", "ffy",
"fia", "fib", "fic", "fie", "fig", "fin", "fir", "fit", "fix", "fka",
"fle", "flo", "flu", "fly", "fmc", "fni", "fob", "foe", "fog", "fon",
"fop", "for", "fox", "fpc", "fra", "fro", "fry", "ftc", "fth", "fty",
"ful", "fum", "fun", "fur", "fus", "gab", "gad", "gae", "gag", "gal",
"gam", "gan", "gao", "gap", "gar", "gas", "gay", "gea", "ged", "gee",
"gei", "gel", "gem", "gen", "ger", "ges", "get", "gey", "gga", "ggs",
"ggy", "ghn", "gho", "ghs", "ght", "ghy", "gia", "gic", "gid", "gie",
"gig", "gil", "gin", "gio", "gis", "git", "gko", "gle", "glo", "gly",
"gma", "gmt", "gmy", "gna", "gne", "gnp", "gnu", "goa", "gob", "god",
"gog", "gol", "gon", "gop", "gor", "gos", "got", "gou", "gow", "gpo",
"gra", "gre", "gro", "gry", "gsa", "gst", "gth", "gua", "gue", "gui",
"gum", "gun", "gur", "gus", "gut", "guy", "gym", "gyp", "hab", "had",
"hae", "hag", "hah", "hai", "hal", "ham", "han", "hao", "hap", "har",
"hat", "haw", "hay", "hby", "hea", "hed", "hee", "hef", "hel", "hem",
"hen", "her", "hes", "het", "hev", "hew", "hex", "hey", "hez", "hia",
"hic", "hid", "hie", "hig", "hil", "him", "hin", "hio", "hip", "hir",
"his", "hit", "hiv", "hiz", "hle", "hly", "hma", "hms", "hna", "hne",
"hns", "hnu", "hoa", "hob", "hoc", "hod", "hoe", "hog", "hoi", "hol",
"hom", "hon", "hoo", "hop", "hor", "hos", "hot", "hou", "how", "hoy",
"hoz", "hra", "hru", "hth", "hty", "hua", "hub", "hud", "hue", "hug",
"huh", "hum", "hun", "hup", "hur", "hus", "hut", "hyl", "iac", "iad",
"iae", "iah", "iak", "ial", "iam", "ian", "iar", "ias", "iat", "iba",
"ibb", "ibe", "ibi", "ibm", "ibn", "ibs", "ica", "icc", "ice", "ich",
"ici", "ick", "ico", "ics", "ict", "icy", "ida", "ide", "idi", "ido",
"idt", "idy", "ied", "ief", "iek", "iel", "iem", "ien", "ier", "ies",
"iet", "ieu", "iev", "iew", "ifa", "ife", "iff", "ifo", "ift", "ify",
"iga", "ige", "igh", "igm", "ign", "igo", "igy", "iii", "ika", "ike",
"iku", "iky", "ila", "ild", "ile", "ili", "ilk", "ill", "ilm", "ilo",
"ilt", "ily", "ima", "imb", "ime", "imh", "imi", "imm", "imo", "imp",
"ims", "imy", "ina", "inc", "ind", "ine", "ing", "inh", "ini", "ink",
"inn", "ino", "ins", "int", "inu", "inx", "iny", "inz", "iod", "iol",
"iom", "ion", "ior", "iot", "ioz", "ipa", "ipe", "iph", "ips", "ipt",
"ira", "ird", "ire", "irk", "irl", "irm", "irn", "iro", "irp", "irs",
"irt", "iry", "isa", "isc", "ise", "ish", "isi", "isk", "ism", "iso",
"isp", "iss", "ist", "isy", "ita", "ite", "ith", "iti", "ito", "itt",
"itu", "ity", "itz", "ium", "ius", "iva", "ive", "ivo", "ivu", "ivy",
"ixt", "ixy", "ize", "jab", "jag", "jah", "jam", "jan", "jar", "jaw",
"jax", "jay", "jed", "jet", "jew", "jig", "jim", "job", "joe", "jog",
"jon", "jor", "jot", "joy", "jug", "jun", "jut", "kah", "kan", "kar",
"kas", "kat", "kay", "ked", "kee", "keg", "kel", "ken", "ker", "kes",
"ket", "kew", "key", "kgo", "kha", "kia", "kid", "kie", "kim", "kin",
"kip", "kit", "kko", "kle", "kok", "kon", "koo", "kor", "kov", "kow",
"kox", "kry", "ksy", "kup", "kus", "kye", "kyo", "lab", "lac", "lad",
"lae", "laf", "lag", "lah", "lai", "lak", "lal", "lam", "lan", "lao",
"lap", "lar", "las", "lat", "lav", "law", "lax", "lay", "lba", "lby",
"lce", "lch", "lco", "lct", "lcy", "lda", "lde", "ldi", "ldo", "lds",
"ldt", "ldy", "lea", "leb", "lec", "led", "lee", "leg", "lei", "lel",
"lem", "len", "leo", "lep", "ler", "les", "let", "lew", "lex", "ley",
"lez", "lfa", "lfe", "lff", "lga", "lge", "lgo", "lhi", "lia", "lib",
"lic", "lid", "lie", "lim", "lin", "lio", "lip", "lis", "lit", "lix",
"liz", "lka", "lke", "lky", "lla", "lle", "lli", "llo", "lls", "lly",
"lma", "lmo", "lmy", "lob", "loc", "lod", "loe", "log", "loh", "loi",
"lom", "lon", "loo", "lop", "lor", "los", "lot", "lou", "lov", "low",
"lox", "loy", "lpa", "lph", "lps", "lpt", "lpy", "lry", "lsa", "lse",
"lsh", "lsi", "lso", "lst", "lsy", "lta", "lte", "lth", "lti", "lto",
"ltv", "lty", "ltz", "lub", "lue", "lug", "lum", "lup", "lur", "lus",
"lut", "lux", "lva", "lve", "lvo", "lye", "lyl", "lyn", "lza", "mac",
"mad", "mae", "mak", "mal", "man", "mao", "map", "mar", "mas", "mat",
"maw", "max", "may", "mba", "mbi", "mbo", "mbu", "mca", "mco", "mea",
"med", "meg", "mel", "men", "meo", "mer", "mes", "met", "mew", "mex",
"mey", "mia", "mic", "mid", "mie", "mig", "mil", "min", "mir", "mis",
"mit", "mix", "mma", "mmo", "mmy", "mna", "mni", "mny", "moa", "mob",
"moe", "mog", "mok", "mom", "mon", "moo", "mop", "mor", "mos", "mot",
"mow", "mox", "mpa", "mpe", "mph", "mpo", "mps", "mpt", "mpy", "mra",
"mrs", "msy", "mth", "mud", "mug", "muk", "mum", "mup", "mur", "mus",
"mut", "nab", "nac", "nad", "nae", "nag", "nah", "nai", "nal", "nam",
"nan", "nao", "nap", "nar", "nas", "nat", "naw", "nay", "nbc", "nbs",
"nca", "nce", "nch", "nck", "nco", "ncr", "nct", "ncy", "nda", "nde",
"ndi", "ndo", "nds", "ndt", "ndu", "ndy", "nea", "neb", "ned", "nee",
"nel", "nen", "neo", "ner", "nes", "net", "new", "nex", "ney", "nez",
"nga", "nge", "ngi", "ngo", "ngs", "ngy", "nia", "nib", "nic", "nid",
"nie", "nig", "nih", "nii", "nik", "nil", "nim", "nin", "nio", "nip",
"nir", "nis", "nit", "nix", "njo", "nka", "nki", "nks", "nky", "nlo",
"nly", "nna", "nne", "nni", "nny", "noa", "nob", "nod", "noe", "noi",
"nol", "nom", "non", "nor", "nos", "not", "nov", "now", "nox", "noy",
"nrc", "nre", "nri", "nry", "nsa", "nse", "nsf", "nsi", "nso", "nst",
"nsy", "nta", "nte", "nth", "nti", "nto", "ntu", "nty", "ntz", "nua",
"nub", "nue", "nug", "nul", "num", "nun", "nuo", "nup", "nus", "nut",
"nvy", "nya", "nyc", "nyl", "nym", "nyu", "nyx", "nza", "nze", "nzy",
"oaa", "oad", "oaf", "oah", "oak", "oal", "oam", "oan", "oap", "oar",
"oat", "oax", "oba", "obb", "obe", "obi", "obo", "obs", "oby", "oca",
"och", "oci", "ock", "oco", "oct", "ocy", "oda", "odd", "ode", "ods",
"ody", "oeb", "oed", "oel", "oem", "oen", "oer", "oes", "oet", "oey",
"ofa", "off", "oft", "ofu", "ofy", "oga", "oge", "ogg", "ogh", "ogi",
"ogo", "ogs", "ogy", "oha", "ohm", "ohn", "ohr", "oia", "oic", "oid",
"oil", "oin", "oir", "ois", "oit", "oix", "oji", "oke", "oki", "oky",
"ola", "old", "ole", "olf", "oli", "olk", "oll", "olm", "oln", "olo",
"ols", "olt", "oly", "oma", "omb", "ome", "omi", "omo", "omp", "omy",
"ona", "ond", "one", "ong", "oni", "onk", "onn", "ono", "ons", "ont",
"onx", "ony", "ood", "oof", "ooh", "ook", "ool", "oom", "oon", "oop",
"oor", "oot", "opa", "ope", "oph", "ops", "opt", "opy", "ora", "orb",
"ord", "ore", "orf", "org", "ori", "ork", "orm", "orn", "oro", "orp",
"orr", "ors", "ort", "ory", "osa", "ose", "osh", "osi", "osk", "osm",
"oso", "oss", "ost", "osy", "ota", "otc", "ote", "oth", "oto", "ott",
"oty", "oud", "oug", "oul", "oum", "oun", "oup", "our", "ous", "out",
"oux", "ova", "ove", "ovo", "ovy", "owa", "owd", "owe", "owl", "own",
"ows", "owy", "oxy", "oya", "oyd", "oyo", "oyt", "oze", "ozy", "pad",
"pal", "pam", "pan", "pap", "par", "pas", "pat", "paw", "pax", "pay",
"paz", "pbs", "pea", "pec", "ped", "pee", "peg", "pei", "pel", "pen",
"pep", "per", "pes", "pet", "pew", "pex", "pey", "pez", "pha", "phd",
"phe", "phi", "phy", "pia", "pic", "pid", "pie", "pig", "pil", "pin",
"pio", "pip", "pir", "pis", "pit", "pla", "ple", "ply", "pod", "poe",
"poi", "pok", "pol", "pon", "poo", "pop", "por", "pos", "pot", "pow",
"pox", "poy", "ppa", "ppe", "ppi", "ppm", "ppo", "pps", "ppy", "pra",
"pro", "pry", "pse", "psi", "pso", "psy", "pta", "pth", "ptu", "pty",
"pua", "pub", "pud", "pug", "pun", "pup", "pur", "pus", "put", "pvc",
"pyl", "qed", "qua", "que", "qui", "quo", "quy", "rab", "rac", "rad",
"rae", "rag", "rah", "raj", "rak", "ral", "ram", "ran", "rap", "raq",
"rar", "ras", "rat", "rau", "raw", "rax", "ray", "rby", "rca", "rce",
"rch", "rck", "rco", "rct", "rcy", "rda", "rde", "rdi", "rdo", "rds",
"rdt", "rdy", "rea", "reb", "red", "ree", "reg", "rei", "rek", "rel",
"rem", "ren", "reo", "rep", "rer", "res", "ret", "rew", "rex", "rey",
"rez", "rge", "rgh", "rgo", "rgy", "rho", "ria", "rib", "ric", "rid",
"rie", "rif", "rig", "ril", "rim", "rin", "rio", "rip", "ris", "rit",
"rix", "rka", "rke", "rks", "rky", "rla", "rld", "rle", "rlo", "rly",
"rma", "rmi", "rmo", "rmy", "rna", "rne", "rno", "rns", "rnt", "rny",
"rob", "rod", "roe", "rof", "rog", "rol", "rom", "ron", "roo", "rop",
"ror", "ros", "rot", "rov", "row", "rox", "roy", "rpa", "rpe", "rph",
"rpm", "rps", "rpt", "rpy", "rra", "rre", "rrh", "rri", "rro", "rry",
"rsa", "rse", "rsh", "rsk", "rso", "rst", "rsy", "rta", "rte", "rth",
"rti", "rto", "rts", "rtt", "rty", "rtz", "rub", "rud", "rue", "rug",
"rum", "run", "rup", "rus", "rut", "rux", "rva", "rve", "rvo", "rvy",
"rye", "ryl", "ryn", "ryo", "rze", "rzo", "sac", "sad", "sae", "saf",
"sag", "sai", "sal", "sam", "san", "sao", "sap", "sar", "sas", "sat",
"sau", "saw", "sax", "say", "sby", "sce", "sch", "scm", "sco", "sda",
"sea", "sec", "sed", "see", "sef", "sel", "sen", "ser", "ses", "set",
"sew", "sex", "sey", "sez", "sfy", "sge", "sgs", "sha", "she", "shi",
"shu", "shy", "sia", "sib", "sic", "sie", "sif", "sik", "sil", "sin",
"sip", "sir", "sis", "sit", "six", "ska", "ske", "ski", "sky", "sle",
"slo", "sly", "sma", "smo", "sna", "sno", "sob", "sod", "soe", "sol",
"som", "son", "sop", "sor", "sou", "sow", "soy", "spa", "sps", "spy",
"sri", "srs", "ssa", "sse", "ssi", "sso", "ssr", "sst", "ssy", "sta",
"std", "ste", "stj", "stm", "sto", "sty", "sub", "sud", "sue", "sul",
"sum", "sun", "sup", "sus", "svp", "tab", "tad", "tae", "tag", "tah",
"tal", "tam", "tan", "tao", "tap", "tar", "tat", "tau", "tav", "taw",
"tax", "tay", "tch", "tcy", "tea", "tec", "ted", "tee", "tel", "tem",
"ten", "teo", "tep", "ter", "tes", "tet", "tew", "tex", "tey", "tha",
"the", "thm", "tho", "thy", "tia", "tic", "tid", "tie", "tif", "tik",
"til", "tim", "tin", "tio", "tip", "tir", "tis", "tit", "tiv", "tka",
"tky", "tle", "tly", "tnt", "toa", "tod", "toe", "tog", "tok", "tol",
"tom", "ton", "too", "top", "tor", "tos", "tot", "tow", "toy", "tra",
"tre", "tri", "tro", "trw", "try", "tsu", "tsy", "tta", "tte", "tti",
"ttl", "tto", "tts", "tty", "tub", "tud", "tue", "tug", "tum", "tun",
"tup", "tur", "tus", "tva", "twa", "two", "twx", "tyl", "tyr", "tyx",
"uac", "uad", "ual", "uam", "uan", "uar", "uat", "uaw", "uay", "uba",
"ube", "ubi", "ubt", "uby", "uce", "uch", "uck", "uct", "ucy", "uda",
"udd", "ude", "udi", "udo", "uds", "udy", "ued", "uee", "uel", "uen",
"uer", "ues", "uet", "uey", "uez", "uff", "uft", "uga", "uge", "ugh",
"ugn", "ugo", "uhn", "uia", "uid", "uil", "uin", "uip", "uir", "uis",
"uit", "uiz", "uji", "uju", "uke", "uki", "ula", "ulb", "uld", "ule",
"ulf", "uli", "ulk", "ull", "ulo", "ulp", "ult", "ulu", "uly", "ulz",
"uma", "umb", "ume", "umn", "ump", "una", "und", "une", "ung", "unk",
"unn", "uno", "unt", "uny", "uod", "uon", "uor", "uot", "uoy", "upe",
"upt", "upy", "ura", "urb", "urd", "ure", "urf", "urg", "uri", "urk",
"url", "urm", "urn", "uro", "urp", "urr", "urs", "urt", "uru", "ury",
"usa", "usc", "use", "ush", "usk", "usn", "uso", "usp", "uss", "ust",
"usy", "ute", "uth", "uti", "uto", "uts", "utt", "utu", "uty", "utz",
"uum", "uve", "uxe", "uyu", "uze", "uzo", "uzz", "vac", "vae", "vah",
"vak", "val", "van", "var", "vas", "vat", "vda", "vea", "ved", "vee",
"veh", "vel", "ven", "ver", "ves", "vet", "vex", "vey", "via", "vic",
"vid", "vie", "vii", "vik", "vil", "vin", "vis", "vit", "viv", "viz",
"voc", "von", "vor", "vot", "vow", "voy", "vre", "vus", "vvy", "wab",
"wac", "wad", "wag", "wah", "wal", "wam", "wan", "wap", "war", "was",
"wat", "wax", "way", "wba", "wca", "wdy", "web", "wed", "wee", "wei",
"wel", "wen", "wer", "wet", "wey", "who", "why", "wie", "wig", "wim",
"win", "wis", "wit", "wky", "wly", "wne", "wns", "wny", "woe", "wok",
"won", "woo", "wop", "wow", "wry", "wse", "wsy", "wth", "wum", "wup",
"wyn", "wzy", "xam", "xan", "xas", "xei", "xel", "xen", "xes", "xia",
"xic", "xie", "xim", "xin", "xis", "xit", "xle", "xom", "xon", "xor",
"xth", "xty", "xup", "xyl", "yad", "yah", "yak", "yal", "yam", "yan",
"yap", "yar", "yaw", "ybe", "yce", "ych", "yda", "yde", "ydn", "yea",
"yed", "yee", "yel", "yen", "yer", "yes", "yet", "yev", "yho", "yin",
"yip", "yke", "yla", "yle", "yli", "yll", "yme", "ymn", "ymy", "yne",
"ynn", "ynx", "yny", "yon", "yor", "you", "yow", "ype", "yph", "ypo",
"ypt", "ypu", "ypy", "yra", "yrd", "yre", "yri", "yro", "yry", "yse",
"yss", "yst", "yta", "yte", "yth", "yuh", "yup", "zac", "zag", "zah",
"zak", "zal", "zan", "zap", "zar", "zda", "zee", "zel", "zen", "zer",
"zet", "zie", "zig", "zil", "zim", "zin", "zip", "zle", "zly", "zoa",
"zoe", "zog", "zon", "zoo", "zor", "zov", "zra", "zza", "zzi", "zzo",
"zzy" ]

    lstr = stir.lower()
    ln = len(lstr)
    if ln < 3:
        return False
    i = bisect.bisect_left(fin, lstr[ln-3:])
    if fin[i] == lstr[ln-3:]:
        return True
    return False


# Copyright Craig Milo Rogers, Jeff turbo Deifik 1988, All rights reserved.
# editdist.c  15 Apr 86  Craig Milo Rogers at USC/ISI Created.
# Rewrote significantly by Jeff turbo Deifik
# Dec-11-2008 translated to python by Jeff turbo Deifik

# This routine calculates the "edit distance" between two strings.  The
# distance metric used is the "Levenshtein distance" with equal weighting for
# insertions, deletions, and substitutions.
#
# For more details on this algorithm see the book "Time Warps, String Edits,
# and Macromolecules: The Theory and Practice of Sequence Comparison", edited
# by David Sankoff and Joseph B. Kruskal.  The basic algorithm is described
# starting on p. 23.  Since we are interested only in the edit distance, and
# not in the edit path, we collapse the reference article's two-dimensional
# array into a one-dimensional array.
#
# The value returned by this routine is the minimum number of  insertions,
# deletions, or substitutions necessary to transform one string to the other.
# The larger the return value, the more the strings differ.  The minimum
# value is 0, the maximum return value is the maximum of the two string
# lengths.

# Note: This is very computationally intensive and is best done in C.
# Ukkonen has some optimizations which make this run a bit faster.
# Applied L.Allison Monash U 92.ILP.html min3 optimization to UKKA

class Editdist:

    # Create the editdist class
    def __init__(self):
        self.__wk = []			# Work area array
        self.__wlen = 0			# Work area array length

    # If you don't have a max_distance
    def Edit_Distance(self, s1, l1, s2, l2):
        return(self.Edit_Dist_With_Max(s1, s1len, s2, s2len, s1len + s2len))

    # This function is similar to Edit_Dist, except that it returns
    # as soon as the distance is equal to or greater than max_distance.
    # For cases where the distance exceeding a certain number is important,
    # this routine will be faster if the real distance is equal to or
    # greater than t.
    # This has the Ukkonen optimization of only considering diagonals of
    # interest. Aka UKKA (with cutoff when central diagonal > max_distance)
    # This is a substantial optimization.
    # Returns the distance count, or max_distance, whichever is smaller.
    # Calculates the edit distance.
    def UKKA_Dist_With_Max(self, s1, s1len, s2, s2len, max_distance):
        if s1len > s2len:		# Exchange s1 and s2, s1len and s2len
            s1, s2 = s2, s1
            s1len, s2len = s2len, s1len

        # Remove common prefix, minimal string copies
        i = 0
        while i < s1len and i < s2len and s1[i] == s2[i]: i += 1
        if i > 0:
            s1 = s1[i:]; s2 = s2[i:]; s1len -= i; s2len -= i

        # Remove common suffix, minimal string copies
        i = 0
        while i < s1len and i < s2len and s1[s1len-i-1] == s2[s2len-i-1]: i += 1
        if i > 0:
            s1 = s1[:-i]; s2 = s2[:-i]; s1len -= i; s2len -= i

        dumbc = 1 + s1len - s2len	# A constant, but compilier doesn't know it.

        # Ukkonen diagonal limits.
                                        # for upper triangle pruning.
        diag_low = -int((max_distance - s2len + s1len)/2)
                                        # for lower triangle pruning.
        diag_high = int((max_distance + s2len - s1len)/2)
        # printf("diag_low is %d, diag_high is %d\n", diag_low, diag_high);

        dumbc = 1 + s1len - s2len	# A constant, but compilier doesn't know it.

    # If the work area big enough, do nothing.
    # Otherwise, make it bigger
        while self.__wlen <= s1len:
            self.__wk.append(0)
            self.__wlen += 1

        for i in range(0, s1len+1):		# Initialize the top margin:
            self.__wk[i] = i

        # The following section of code is very tricky.  The work array and the
        # strings are indexed starting at 0, but str[0] really corresponds to
        # wk[1].
        for j in range(0, s2len):		# Scan the second string.
            # lower and upper are the starting and ending bounds for the inner loop
            lower = max(1, j+1 - diag_high)
            upper = min(s1len, j+1 - diag_low)

            prv = self.__wk[lower-1]		# Keep previous value.
            self.__wk[lower-1] = j+1		# Marginal weight.

            # If I wasn't using the Ukkonen optimization, this loop would be
            # for i in range (1, s1len)
            # The Ukkonen optimization turns the complexity from
            # O(s1len*s2len) to O(s1len*max_distance)

            for i in range(lower, upper+1):	# Scan the first string.
                if s2[j] == s1[i-1]:		# Test for a substitution.
                    cost_s = prv		# No substitution, cheapest path.
                else:
                    cost_s = prv + 1		# Weight for a substitution.

                cost_i = self.__wk[i] + 1	# Weight for an insertion.

# This is the sneaky min3 optimization from
# http://www.csse.monash.edu.au/~lloyd/tildeStrings/Alignment/92.IPL.html
                if cost_i < cost_s:
                    mn = cost_i			# This is min
                else:
                    cost_d = self.__wk[i-1] + 1	# Weight for a deletion.
# The below line was expanded in the interest of efficiency.
# Since python doesn't have inline functions, I save a function call.
# Additionally, my min takes in exactly 2 arguments, so there is no
# need for a loop to loop over the input to find min
# About 6% faster!
#                    mn = min(cost_s, cost_d)	# This is min
                    if cost_s < cost_d: mn = cost_s
                    else: mn = cost_d

                prv = self.__wk[i]		# Save cell's previous value.
                self.__wk[i] = mn		# Store min distance

                # If valid diagonal and distance is too big.
            j_plus_dumbc = j + dumbc
            if j_plus_dumbc >= 0 and self.__wk[j_plus_dumbc] >= max_distance:
                return(max_distance)

        return (self.__wk[s1len])		# Return the final distance.

    # This function is similar to Edit_Distance, except that it returns
    # as soon as the distance is equal to or greater than max_distance.
    # For cases where the distance exceeding a certain number is important,
    # this routine will be faster if the real distance is equal to or
    # greater than max_distance.
    # Returns the distance count, or max_distance, whichever is smaller.
    # Calculates the edit distance.

    def Edit_Dist_With_Max(self, s1, s1len, s2, s2len, max_distance):
        if s1len > s2len:		# Exchange s1 and s2, s1len and s2len
            s1, s2 = s2, s1
            s1len, s2len = s2len, s1len

        dumbc = 1 + s1len - s2len;	# A constant, but compilier doesn't know it.

    # If the work area big enough, do nothing.
    # Otherwise, make it bigger
        while self.__wlen <= s1len:
            self.__wk.append(0)
            self.__wlen += 1

        for i in range(0, s1len+1):		# Initialize the top margin:
            self.__wk[i] = i

        # The following section of code is very tricky.  The work array and the
        # strings are indexed starting at 0, but s[0] really corresponds to wk[1].

        for j in range(0, s2len):		# Scan the second string.
            prv = self.__wk[0]			# Keep previous value.
            self.__wk[0] = j+1			# Marginal weight.

            for i in range(1, s1len+1):		# Scan the first string.
                if s2[j] == s1[i-1]:		# Test for a substitution.
                    d = prv			# No substitution, cheapest path.
                else:
                    d = prv + 1			# Weight for a substitution.

                prv = self.__wk[i]		# Save cell's previous value.
                if d > (self.__wk[i] + 1):
                    d = self.__wk[i] + 1	# Weight for an insertion.
                if d > (self.__wk[i-1] + 1):
                    d = self.__wk[i-1] + 1	# Weight for a deletion.
                self.__wk[i] = d		# Store the distance so far.

            # If valid diagonal and distance is too big.
            j_plus_dumbc = j + dumbc
            if j_plus_dumbc >= 0 and self.__wk[j_plus_dumbc] >= max_distance:
                return(max_distance)

        return(self.__wk[s1len])		# Return the final distance.
