# $Id: jeff_ncomment.py,v 1.4 2012-10-03 01:19:47 jdeifik Exp $
# Copyright Jeff turbo Deifik 2008
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# jeff_ncomment python class
# Author:	Jeff Deifik Sep-23-2008
# Bugs:	For c, 'escape characters' may not be recognized.


from __future__ import print_function
import jeff_char


class jeff_ncomment:


    # Create the jeff_ncomment class
    # Lang must be one of the following:
    # 'c', 'braces', 'parens', 'old_pascal', 'new_pascal', 'fortran', 'ada'
    # 'macro', 'shell', 'python'
    def __init__(self, lang):
        self.__lang = lang
        self.__ccom = False		# State for multi-line C comments
        self.__pcom = False		# State for multi-line Pascal comments
        self.__xcom = 0			# State for multi-line Pascal comments
    # Make a dictionary and call it switch
        self.__switch = {
            'c':	 	self.__ccomment,
            'shell': 		self.__scomment,
            'python':     	self.__scomment,
            'braces':     	self.__bcomment,
            'parens':     	self.__pcomment,
            'old_pascal': 	self.__ocomment,
            'new_pascal': 	self.__wcomment,
            'fortran':    	self.__fcomment,
            'ada':        	self.__acomment,
            'macro':      	self.__mcomment
        }


    # Does the file copying.
    def do_line(self, stri):
        stri = self.__switch[self.__lang](stri)
        stri = jeff_char.Str_Tblank(stri)		# Remove trailing blanks.
        return stri


    # Process quotes and double quotes on a line by line basis
    # Takes in ch - the current character being processed,
    # incomment - true when in a comment,
    # qdouble - true when in a double quote,
    # qsingle - true when in a single quote
    # Returns (qdouble, qsingle) set appropriately
    def process_quotes(self, ch, incomment, qdouble, qsingle):
        dquo = '"'
        quo  = "'"
        if not incomment:
            if ch == dquo and not qdouble:  qdouble = True
            elif ch == dquo and qdouble: qdouble = False
            elif ch == quo and not qsingle: qsingle = True
            elif ch == quo and qsingle:  qsingle = False
        return (qdouble, qsingle)


    # Remove C and C++ comments.
    # Find a // not in a quoted strong, and remove everything to end-of-line
    def __ccomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], self.__ccom,
                                                       qdouble, qsingle)
            if qsingle == False and qdouble == False:
                # Deal with c++ style comments //
                if i < ln-1 and t[i:i+2] == '//':
                    return t[0:i]
                if i < ln-1 and t[i:i+2]=='/*':
                    self.__ccom = True
                elif i < ln-1 and t[i:i+2]=='*/':
                    self.__ccom = False
                    t = t[0:i] + '  ' + t[i+2:]
                if self.__ccom:
                    t = t[0:i] + ' ' + t[i+1:]
        return t


    # Turn Pascal comments { ... } into spaces.
    def __bcomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], self.__pcom,
                                                       qdouble, qsingle)
            if not qsingle and not qdouble:
                if t[i]=='{':
                    self.__pcom = True
                elif t[i]=='}':
                    self.__pcom = False
                    t = t[0:i] + ' ' + t[i+1:]
                if self.__pcom:
                    t = t[0:i] + ' ' + t[i+1:]
        return t


    # Turn Pascal comments (* ... *) into spaces.
    def __pcomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], self.__pcom,
                                                       qdouble, qsingle)
            if not qsingle and not qdouble:
                if i < ln-1 and t[i:i+2]=='(*':
                    self.__pcom = True
                elif i < ln-1 and t[i:i+2]=='*)':
                    self.__pcom = False
                    t = t[0:i] + '  ' + t[i+2:]
                if self.__pcom:
                    t = t[0:i] + ' ' + t[i+1:]
        return t


    # Turn Pascal comments (* ... *) or { ... } into spaces.
    def __ocomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], self.__xcom,
                                                       qdouble, qsingle)
            if not qsingle and not qdouble:
                if self.__xcom == 0 and t[i]=='{':		# for {
                    self.__xcom = 1
                elif self.__xcom == 1 and t[i]=='}':		# for }
                    self.__xcom = 0
                    t = t[0:i] + ' ' + t[i+1:]
                if self.__xcom == 0 and i < ln-1 and t[i:i+2]=='(*':
                    self.__xcom = 2
                elif self.__xcom == 2 and i < ln-1 and t[i:i+2]=='*)':
                    self.__xcom = 0
                    t = t[0:i] + '  ' + t[i+2:]
                if self.__xcom != 0:
                    t = t[0:i] + ' ' + t[i+1:]
        return t


    # Turn Pascal comments (* or { ... } or *) into spaces.
    # Nesting of comments is not allowed.
    def __wcomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], self.__pcom,
                                                       qdouble, qsingle)
            if not qsingle and not qdouble:
                if t[i]=='{':				# for {
                    self.__pcom = True
                elif t[i]=='}':				# for }
                    self.__pcom = False
                    t = t[0:i] + ' ' + t[i+1:]
                if i < ln-1 and t[i:i+2]=='(*':
                    self.__pcom = True
                elif i < ln-1 and t[i:i+2]=='*)':
                    self.__pcom = False
                    t = t[0:i] + '  ' + t[i+2:]
                if self.__pcom:
                    t = t[0:i] + ' ' + t[i+1:]
        return t


    # Remove Fortran comments.
    # Find a C at the start of the line, and remove everything to end-of-line
    def __fcomment(self, t):
        if len(t) > 0:			# Remove comment
            if t[0].tolower() == 'c':
                return t[0:0]
        return t


    # Remove Ada comments.
    # Find a -- not in a quoted strong, and remove everything to end-of-line
    def __acomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], False,
                                                       qdouble, qsingle)
            if not qsingle and not qdouble:
                if i < ln-1 and t[i:i+2] == '--':
                    return t[0:i]
        return t


    # Remove vax macro comments.
    # Find a ; not in a quoted strong, and remove everything to end-of-line
    def __mcomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], False,
                                                       qdouble, qsingle)
            if qsingle == False and qdouble == False:
                if t[i] == ';':
                    return t[0:i]
        return t


    # Remove shell comments.
    # Find a # not in a quoted strong, and remove everything to end-of-line
    def __scomment(self, t):
        qsingle = False
        qdouble = False

        ln = len(t)
        for i in range(0, ln):		# Remove comment
            (qdouble, qsingle) = self.process_quotes(t[i], False,
                                                       qdouble, qsingle)
            if qsingle == False and qdouble == False:
                if t[i] == '#':
                    return t[0:i]
        return t
